﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Analysis.SolutionCaching;
using HeuristicLab.Analysis.SolutionCaching.PermutationEncoding;
using HeuristicLab.Encodings.PermutationEncoding;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace AlgorithmBehaviorUnitTests {
  [TestClass]
  public class PermutationCacheTest {
    [TestMethod]
    public void TestPermutationSolutionDictionary() {
      var rand = new HeuristicLab.Random.FastRandom();
      PermutationEqualityComparer comp = new PermutationEqualityComparer();
      PermutationSolutionCache dict = new PermutationSolutionCache();
      int nrOfPermutations = 200;


      for (int i = 0; i < nrOfPermutations; i++) {
        var p = new Permutation(PermutationTypes.RelativeUndirected, 21, rand);

        //should be added
        PermutationSolutionInformation pi = new PermutationSolutionInformation();
        pi.Iteration = 9;
        pi.ProducedBy = ProducedBy.Mutation;
        dict.Add(p, pi);

        //should be added
        pi = new PermutationSolutionInformation();
        pi.Iteration = 10;
        pi.ProducedBy = ProducedBy.Mutation;
        dict.Add(p, pi);

        //should be added
        pi = new PermutationSolutionInformation();
        pi.Iteration = 10;
        pi.ProducedBy = ProducedBy.Crossover;
        dict.Add(p, pi);

        //should not be added
        pi = new PermutationSolutionInformation();
        pi.Iteration = 10;
        pi.ProducedBy = ProducedBy.Mutation;
        dict.Add(p, pi);
      }

      foreach (var value in dict.Values()) {
        Assert.AreEqual(3, value.Count);
      }

      foreach (var key in dict.Keys()) {
        int cnt = dict.Keys().Count(x => comp.Equals(key, x));
        Assert.AreEqual(1, cnt);
      }
    }
  }
}
